"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CachedStorage = void 0;
const cache_error_1 = require("./cache/cache.error");
class CachedStorage {
    constructor(storage, cache, options) {
        this.storage = storage;
        this.cache = cache;
        this.options = options;
        this.readonly = false;
        this.DEFAULT_OPTIONS = {
            expire: 3600,
            prefix: 'IACRY_PCACHE/',
        };
        if (!options) {
            this.options = this.DEFAULT_OPTIONS;
        }
        this.options = !options
            ? this.DEFAULT_OPTIONS
            : Object.assign({}, this.DEFAULT_OPTIONS, this.options);
    }
    async fetch(principal) {
        const key = this.key(principal);
        if (await this.cache.has(key)) {
            return this.decode(await this.cache.get(key));
        }
        const policies = await this.storage.fetch(principal);
        if (!(await this.cache.set(key, this.encode(policies), this.options.expire))) {
            throw new cache_error_1.CacheError(`Unable to store policy cache for key: ${key}`);
        }
        return policies;
    }
    async save(principal, rawPolicies) {
        await this.purgeCache(principal);
        return this.storage.save(principal, rawPolicies);
    }
    async add(principal, rawPolicies) {
        await this.purgeCache(principal);
        return this.storage.add(principal, rawPolicies);
    }
    async purge(principal) {
        await this.purgeCache(principal);
        return this.storage.purge(principal);
    }
    async fetchBySid(sid, principal) {
        const key = this.key(principal, sid);
        if (await this.cache.has(key)) {
            return this.decode(await this.cache.get(key));
        }
        const policies = await this.storage.fetchBySid(sid, principal);
        if (!(await this.cache.set(key, this.encode(policies), this.options.expire))) {
            throw new cache_error_1.CacheError(`Unable to store policy cache for key: ${key}`);
        }
        return policies;
    }
    async saveBySid(sid, principal, rawPolicies) {
        await Promise.all([
            this.purgeCache(principal),
            this.purgeCache(principal, sid),
        ]);
        return this.storage.saveBySid(sid, principal, rawPolicies);
    }
    async purgeCache(principal, sid) {
        const key = this.key(principal, sid);
        if (await this.cache.has(key)) {
            if (!(await this.cache.remove(key))) {
                throw new cache_error_1.CacheError(`Unable to remove policy cache for key: ${key}`);
            }
        }
    }
    encode(policies) {
        return JSON.stringify(policies);
    }
    decode(rawPolicies) {
        return JSON.parse(rawPolicies);
    }
    key(principal, sid) {
        const key = `${this.options.prefix}${principal.entity}/${principal.id}`;
        return sid ? `${key}/${sid}` : key;
    }
}
exports.CachedStorage = CachedStorage;
